@@ -17,7 +17,7 @@ from dataclasses import dataclass, field
 
 from beartype.typing import Any, Sequence
 
-from superlinked.framework.common.storage.search_index_creation.index_field_descriptor import (
+from superlinked.framework.common.storage.search_index.index_field_descriptor import (
     IndexFieldDescriptor,
     VectorIndexFieldDescriptor,
 )
@@ -31,9 +31,7 @@ class IndexConfig:
     index_params: dict[str, Any] = field(default_factory=dict)
 
     def __post_init__(self) -> None:
-        self.__indexed_field_names = [
-            field_descriptor.field_name for field_descriptor in self.field_descriptors
-        ]
+        self.__indexed_field_names = [field_descriptor.field_name for field_descriptor in self.field_descriptors]
 
     @property
     def indexed_field_names(self) -> Sequence[str]:
