@@ -18,21 +18,24 @@ from beartype.typing import Any, Sequence
 
 from superlinked.framework.common.calculation.distance_metric import DistanceMetric
 from superlinked.framework.common.const import constants
-from superlinked.framework.common.exception import ValidationException
-from superlinked.framework.common.storage.entity import Entity
-from superlinked.framework.common.storage.entity_data import EntityData
-from superlinked.framework.common.storage.field import Field
+from superlinked.framework.common.exception import InitializationException
+from superlinked.framework.common.storage.entity.entity import Entity
+from superlinked.framework.common.storage.entity.entity_data import EntityData
 from superlinked.framework.common.storage.index_config import IndexConfig
 from superlinked.framework.common.storage.query.vdb_knn_search_params import (
     VDBKNNSearchParams,
 )
 from superlinked.framework.common.storage.result_entity_data import ResultEntityData
-from superlinked.framework.common.storage.search_index_creation.search_algorithm import (
+from superlinked.framework.common.storage.search_index.manager.search_index_manager import (
+    SearchIndexManager,
+)
+from superlinked.framework.common.storage.search_index.search_algorithm import (
     SearchAlgorithm,
 )
-from superlinked.framework.common.storage.search_index_creation.vector_component_precision import (
+from superlinked.framework.common.storage.search_index.vector_component_precision import (
     VectorComponentPrecision,
 )
+from superlinked.framework.common.util.string_util import StringUtil
 from superlinked.framework.storage.in_memory.object_serializer import ObjectSerializer
 
 
@@ -48,9 +51,9 @@ class VDBConnector(ABC):
         self._search_algorithm = search_algorithm
         self._vector_coordinate_type = vector_coordinate_type
         self._index_configs: dict[str, IndexConfig] = {
-            index_config.index_name: index_config
-            for index_config in (index_configs or [])
+            index_config.index_name: index_config for index_config in (index_configs or [])
         }
+        self._app_id = self._generate_app_id(index_configs or [])
 
     @property
     def distance_metric(self) -> DistanceMetric:
@@ -64,13 +67,23 @@ class VDBConnector(ABC):
     def vector_coordinate_type(self) -> VectorComponentPrecision:
         return self._vector_coordinate_type
 
+    @property
+    def collection_name(self) -> str:
+        if self._app_id is None:
+            raise InitializationException(
+                "app id wasn't initialized properly by calling "
+                + "by either initializing the vdb onnector with a set of index configs "
+                + "or calling init_search_index_configs with them."
+            )
+        return self._app_id
+
     @abstractmethod
     def close_connection(self) -> None:
         pass
 
     @property
     @abstractmethod
-    def supported_vector_indexing(self) -> Sequence[SearchAlgorithm]:
+    def search_index_manager(self) -> SearchIndexManager:
         pass
 
     @property
@@ -78,10 +91,6 @@ class VDBConnector(ABC):
     def _default_search_limit(self) -> int:
         pass
 
-    @abstractmethod
-    def _list_search_index_names_from_vdb(self) -> Sequence[str]:
-        pass
-
     def persist(self, _: ObjectSerializer) -> None:
         """
         Persist the state of the VDB. Implement this method in subclasses if persistence is supported.
@@ -92,38 +101,23 @@ class VDBConnector(ABC):
         Restore the state of the VDB. Implement this method in subclasses if restoration is supported.
         """
 
+    def _generate_app_id(self, index_configs: Sequence[IndexConfig]) -> str | None:
+        hash_ = StringUtil.deterministic_hash_of_strings([index_config.index_name for index_config in index_configs])
+        return hash_ if hash_ else None
+
     def init_search_index_configs(
         self,
         index_configs: Sequence[IndexConfig],
+        create_search_indices: bool,
         override_existing: bool = False,
     ) -> None:
-        existing_index_names = self._list_search_index_names_from_vdb()
-        for index_config in index_configs:
-            if index_config.index_name not in existing_index_names or override_existing:
-                self.create_search_index_with_check(index_config)
-            else:
-                self._index_configs[index_config.index_name] = index_config
-
-    def create_search_index_with_check(self, index_config: IndexConfig) -> None:
-        if index_config.index_name not in self._index_configs.keys():
-            if (
-                index_config.vector_field_descriptor.search_algorithm
-                not in self.supported_vector_indexing
-            ):
-                raise NotImplementedError(
-                    f"The specified vector search algorithm {index_config.vector_field_descriptor.search_algorithm}"
-                    + f" is not yet supported. Currently supported algorithms: {self.supported_vector_indexing}"
-                )
-            self.create_search_index(index_config)
-            self._index_configs[index_config.index_name] = index_config
-
-    @abstractmethod
-    def create_search_index(self, index_config: IndexConfig) -> None:
-        pass
-
-    @abstractmethod
-    def drop_search_index(self, index_name: str) -> None:
-        pass
+        self._app_id = self._generate_app_id(index_configs)
+        self.search_index_manager.init_search_indices(
+            index_configs,
+            self.collection_name,
+            create_search_indices,
+            override_existing,
+        )
 
     @abstractmethod
     def write_entities(self, entity_data: Sequence[EntityData]) -> None:
@@ -137,7 +131,6 @@ class VDBConnector(ABC):
         self,
         index_name: str,
         schema_name: str,
-        returned_fields: Sequence[Field],
         vdb_knn_search_params: VDBKNNSearchParams,
         **params: Any,
     ) -> Sequence[ResultEntityData]:
@@ -150,28 +143,21 @@ class VDBConnector(ABC):
         search_params = VDBKNNSearchParams(
             vector_field=vdb_knn_search_params.vector_field,
             limit=limit,
+            fields_to_return=vdb_knn_search_params.fields_to_return,
             filters=vdb_knn_search_params.filters,
             radius=vdb_knn_search_params.radius,
         )
-        return self._knn_search(
-            index_name, schema_name, returned_fields, search_params, **params
-        )
+        return self._knn_search(index_name, schema_name, search_params, **params)
 
     @abstractmethod
     def _knn_search(
         self,
         index_name: str,
         schema_name: str,
-        returned_fields: Sequence[Field],
         vdb_knn_search_params: VDBKNNSearchParams,
         **params: Any,
     ) -> Sequence[ResultEntityData]:
         pass
 
     def _get_index_config(self, index_name: str) -> IndexConfig:
-        index_config = self._index_configs.get(index_name)
-        if not index_config:
-            raise ValidationException(
-                f"Index with the given name {index_name} doesn't exist!"
-            )
-        return index_config
+        return self.search_index_manager.get_index_config(index_name)
