@@ -12,36 +12,41 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import math
 
-from beartype.typing import Mapping, cast
 from typing_extensions import override
 
 from superlinked.framework.common.dag.constant_node import ConstantNode
-from superlinked.framework.common.dag.node import Node
-from superlinked.framework.common.dag.number_embedding_node import (
-    NumberEmbeddingNode,
-    NumberEmbeddingParams,
-)
+from superlinked.framework.common.dag.embedding_node import EmbeddingNode
+from superlinked.framework.common.dag.number_embedding_node import NumberEmbeddingNode
 from superlinked.framework.common.dag.schema_field_node import SchemaFieldNode
-from superlinked.framework.common.data_types import Vector
-from superlinked.framework.common.embedding.number_embedding import (
+from superlinked.framework.common.schema.schema_object import Number, SchemaObject
+from superlinked.framework.common.space.config.aggregation.aggregation_config import (
+    AggregationConfig,
+    AvgAggregationConfig,
+    MaxAggregationConfig,
+    MinAggregationConfig,
+)
+from superlinked.framework.common.space.config.embedding.number_embedding_config import (
     LinearScale,
-    LogarithmicScale,
     Mode,
+    NumberEmbeddingConfig,
     Scale,
 )
-from superlinked.framework.common.schema.schema_object import Number, SchemaObject
-from superlinked.framework.common.space.aggregation import InputAggregationMode
-from superlinked.framework.dsl.space.exception import (
-    InvalidSpaceParamException,
-    NoDefaultNodeException,
+from superlinked.framework.common.space.config.normalization.normalization_config import (
+    NoNormConfig,
 )
+from superlinked.framework.common.space.config.transformation_config import (
+    TransformationConfig,
+)
+from superlinked.framework.common.util.lazy_property import lazy_property
+from superlinked.framework.dsl.space.exception import NoDefaultNodeException
+from superlinked.framework.dsl.space.has_space_field_set import HasSpaceFieldSet
+from superlinked.framework.dsl.space.input_aggregation_mode import InputAggregationMode
 from superlinked.framework.dsl.space.space import Space
 from superlinked.framework.dsl.space.space_field_set import SpaceFieldSet
 
 
-class NumberSpace(Space):
+class NumberSpace(Space[float, float], HasSpaceFieldSet):
     """
     NumberSpace is used to encode numerical values within a specified range.
     The range is defined by the min_value and max_value parameters.
@@ -67,12 +72,6 @@ class NumberSpace(Space):
             Possible values are: maximum, minimum and average.
         negative_filter (float): This is a value that will be set for everything that is equal or
             lower than the min_value. It can be a float. It defaults to 0 (No effect)
-
-    Raises:
-        InvalidSpaceParamException: If multiple fields of the same schema are in the same space.
-            Or the min_value is bigger than the max value, or the negative filter bigger than 0
-        InvalidSchemaException: If there's no node corresponding to a given schema.
-
     """
 
     def __init__(  # pylint: disable=too-many-arguments
@@ -106,134 +105,133 @@ class NumberSpace(Space):
                 Possible values are: maximum, minimum and average.
             negative_filter (float): This is a value that will be set for everything that is equal or
                 lower than the min_value. It can be a float. It defaults to 0 (No effect)
-
-         Raises:
-            InvalidSpaceParamException: If multiple fields of the same schema are in the same space.
-                Or the min_value is bigger than the max value, or the negative filter bigger than 0
-            InvalidSchemaException: If there's no node corresponding to a given schema.
         """
-        self.aggregation_mode = aggregation_mode
-        super().__init__(number, Number)
-        self.__validate_parameters(min_value, max_value, negative_filter)
-        self.embedding_params = NumberEmbeddingParams(
-            min_value=float(min_value),
-            max_value=float(max_value),
-            mode=mode,
-            scale=scale,
-            negative_filter=negative_filter,
+        self._aggregation_mode = aggregation_mode  # this must be set before super init for _handle_node_not_present
+        self._embedding_config = NumberEmbeddingConfig(
+            float,
+            float(min_value),
+            float(max_value),
+            mode,
+            scale,
+            negative_filter,
         )
-        self.default_constant_node_input: int | float | None
-        match mode:
-            case Mode.MAXIMUM:
-                self.default_constant_node_input = max_value
-            case Mode.MINIMUM:
-                self.default_constant_node_input = min_value
-            case Mode.SIMILAR:
-                self.default_constant_node_input = None
-            case _:
-                raise ValueError(f"Unknown mode: {mode}")
-        number_node_map = {
-            num: NumberEmbeddingNode(
-                SchemaFieldNode(num), self.embedding_params, self.aggregation_mode
+        super().__init__(number, Number)
+        number_fields = number if isinstance(number, list) else [number]
+        self.number = SpaceFieldSet[float](self, set(number_fields))
+        self._aggregation_config_type_by_mode = self.__init_aggregation_config_type_by_mode()
+        self._transformation_config = self._init_transformation_config(self._embedding_config, self._aggregation_mode)
+        self.__schema_node_map: dict[SchemaObject, EmbeddingNode[float, float]] = {
+            number_field.schema_obj: NumberEmbeddingNode(
+                parent=SchemaFieldNode(number_field),
+                transformation_config=self._transformation_config,
+                fields_for_identification=self.number.fields,
             )
-            for num in self._field_set
-        }
-        self.number = SpaceFieldSet(self, set(number_node_map.keys()))
-        self.__schema_node_map: dict[SchemaObject, Node] = {
-            schema_field.schema_obj: node
-            for schema_field, node in number_node_map.items()
+            for number_field in number_fields
         }
 
     @property
-    def _node_by_schema(self) -> Mapping[SchemaObject, Node[Vector]]:
+    @override
+    def _embedding_node_by_schema(
+        self,
+    ) -> dict[SchemaObject, EmbeddingNode[float, float]]:
         return self.__schema_node_map
 
     @property
     @override
-    def annotation(self) -> str:
-        mode_text = self.embedding_params.mode.value
+    def space_field_set(self) -> SpaceFieldSet:
+        return self.number
+
+    @property
+    @override
+    def transformation_config(self) -> TransformationConfig[float, float]:
+        return self._transformation_config
+
+    def __init_aggregation_config_type_by_mode(
+        self,
+    ) -> dict[InputAggregationMode, type[AggregationConfig]]:
+        return {
+            InputAggregationMode.INPUT_AVERAGE: AvgAggregationConfig,
+            InputAggregationMode.INPUT_MINIMUM: MinAggregationConfig,
+            InputAggregationMode.INPUT_MAXIMUM: MaxAggregationConfig,
+        }
+
+    @property
+    @override
+    def _annotation(self) -> str:
+        mode_text = self._embedding_config.mode.value
         mode_to_preference: dict[str, str] = {
             "minimum": "lower",
             "maximum": "higher",
             "similar": "similar",
         }
         similar_first_text = (
-            """
-            s to the one supplied in a .similar clause during a Query.
-            """
-            if self.embedding_params.mode == Mode.SIMILAR
+            " to the one supplied in a .similar clause during a Query"
+            if self._embedding_config.mode == Mode.SIMILAR
             else ""
         )
         negative_text: dict[str, str] = {
-            "similar": "the values further from the value in the corresponding mandatory similar clause.",
+            "similar": "the values further from the value in the corresponding mandatory similar clause",
             "minimum": "higher values",
             "maximum": "lower values",
         }
         end_text: str = (
             " Accepts int or float type input for a corresponding .similar clause input."
-            if self.embedding_params.mode == Mode.SIMILAR
+            if self._embedding_config.mode == Mode.SIMILAR
             else ""
         )
-        scaling_text = (
-            f"logarithmically with the base of {self.embedding_params.scale.base}"
-            if isinstance(self.embedding_params.scale, LogarithmicScale)
-            else "linearly"
-        )
-        min_value = (
-            math.log(
-                1 + self.embedding_params.min_value,
-                self.embedding_params.scale.base,
-            )
-            if isinstance(self.embedding_params.scale, LogarithmicScale)
-            else self.embedding_params.min_value
-        )
-        max_value = (
-            math.log(
-                1 + self.embedding_params.max_value,
-                self.embedding_params.scale.base,
-            )
-            if isinstance(self.embedding_params.scale, LogarithmicScale)
-            else self.embedding_params.max_value
-        )
+        min_value = self._embedding_config.min_value
+        max_value = self._embedding_config.max_value
+
         return f"""The space encodes numbers between {min_value}
         and {max_value}, being the domain of the space.
-        Values are {scaling_text} spaced in {min_value} and {max_value}.
+        Values are spaced between {min_value} and {max_value}.
         It has {mode_text} Mode so it favors the {mode_to_preference[mode_text]} number{similar_first_text}.
         For this {mode_text} mode space, negative weights mean favoring
         the {negative_text[mode_text]}. 0 weight means insensitivity.
-        Larger positive weights increase the effect on similarity compared to other spaces. Space weights do not matter
-        if there is only 1 space in the query.{end_text}"""
+        Larger positive weights increase the effect on similarity compared to other spaces.{end_text}
+        Affected fields: {self.space_field_set.field_names_text}."""
 
     @property
     @override
     def _allow_empty_fields(self) -> bool:
-        return self.aggregation_mode == Mode.SIMILAR
+        return self._embedding_config.mode == Mode.SIMILAR
 
-    def __validate_parameters(
-        self, min_value: float | int, max_value: float | int, negative_filter: float
-    ) -> None:
-        if min_value >= max_value:
-            raise InvalidSpaceParamException(
-                f"The maximum value ({max_value}) should be greater than the minimum value ({min_value})."
-            )
-        if negative_filter > 0:
-            raise InvalidSpaceParamException(
-                f"The negative filter value should not be more than 0. Value is: {negative_filter}"
-            )
+    @property
+    @override
+    def allow_similar_clause(self) -> bool:
+        return self._embedding_config.mode == Mode.SIMILAR
+
+    def _init_transformation_config(
+        self,
+        embedding_config: NumberEmbeddingConfig,
+        aggregation_mode: InputAggregationMode,
+    ) -> TransformationConfig[float, float]:
+        aggregation_config = self._aggregation_config_type_by_mode[aggregation_mode](float)
+        normalization_config = NoNormConfig()
+        return TransformationConfig(normalization_config, aggregation_config, embedding_config)
 
     @override
-    def _handle_node_not_present(self, schema: SchemaObject) -> NumberEmbeddingNode:
-        if self.embedding_params.mode is Mode.SIMILAR:
-            raise NoDefaultNodeException(
-                "Number Space with SIMILAR Mode do not have a default value, a .similar "
-                "clause is needed in the query."
-            )
-        constant_node = cast(
-            Node, ConstantNode(value=self.default_constant_node_input, schema=schema)
+    def _create_default_node(self, schema: SchemaObject) -> EmbeddingNode[float, float]:
+        constant_node = ConstantNode(value=self._default_constant_node_input, schema=schema)
+        default_node = NumberEmbeddingNode(
+            parent=constant_node,
+            transformation_config=self._transformation_config,
+            fields_for_identification=self.number.fields,
         )
+        return default_node
 
-        number_embedding_node = NumberEmbeddingNode(
-            constant_node, self.embedding_params, self.aggregation_mode
-        )
-        self.__schema_node_map[schema] = number_embedding_node
-        return number_embedding_node
+    @lazy_property
+    def _default_constant_node_input(self) -> float:
+        match self._embedding_config.mode:
+            case Mode.MAXIMUM:
+                default_constant_node_input = self._embedding_config.max_value
+            case Mode.MINIMUM:
+                default_constant_node_input = self._embedding_config.min_value
+            case Mode.SIMILAR:
+                raise NoDefaultNodeException(
+                    "Number Space with SIMILAR Mode do not have a default value, a .similar "
+                    "clause is needed in the query."
+                )
+            case _:
+                raise ValueError(f"Unknown mode: {self._embedding_config.mode}")
+        return default_constant_node_input
