@@ -12,21 +12,37 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import math
 from datetime import timedelta
 
 import structlog
-from beartype.typing import Mapping
+from beartype.typing import Sequence
 from typing_extensions import override
 
+from superlinked.framework.common.dag.embedding_node import EmbeddingNode
 from superlinked.framework.common.dag.named_function_node import NamedFunctionNode
-from superlinked.framework.common.dag.node import Node
 from superlinked.framework.common.dag.period_time import PeriodTime
 from superlinked.framework.common.dag.recency_node import RecencyNode
 from superlinked.framework.common.dag.schema_field_node import SchemaFieldNode
-from superlinked.framework.common.data_types import Vector
 from superlinked.framework.common.schema.schema_object import SchemaObject, Timestamp
-from superlinked.framework.common.space.aggregation import InputAggregationMode
+from superlinked.framework.common.space.config.aggregation.aggregation_config import (
+    AggregationConfig,
+    AvgAggregationConfig,
+    MaxAggregationConfig,
+    MinAggregationConfig,
+)
+from superlinked.framework.common.space.config.embedding.recency_embedding_config import (
+    RecencyEmbeddingConfig,
+)
+from superlinked.framework.common.space.config.normalization.normalization_config import (
+    ConstantNormConfig,
+)
+from superlinked.framework.common.space.config.transformation_config import (
+    TransformationConfig,
+)
 from superlinked.framework.common.util.named_function_evaluator import NamedFunction
+from superlinked.framework.dsl.space.has_space_field_set import HasSpaceFieldSet
+from superlinked.framework.dsl.space.input_aggregation_mode import InputAggregationMode
 from superlinked.framework.dsl.space.space import Space
 from superlinked.framework.dsl.space.space_field_set import SpaceFieldSet
 
@@ -35,7 +51,7 @@ logger = structlog.getLogger()
 DEFAULT_PERIOD_TIME = PeriodTime(period_time=timedelta(days=14))
 
 
-class RecencySpace(Space):  # pylint: disable=too-many-instance-attributes
+class RecencySpace(Space[int, int], HasSpaceFieldSet):  # pylint: disable=too-many-instance-attributes
     """
     Recency space encodes timestamp type data measured in seconds and in unix timestamp format.
     Recency space is utilized to encode how recent items are. Use period_time_list
@@ -84,50 +100,71 @@ class RecencySpace(Space):  # pylint: disable=too-many-instance-attributes
                 Day will be the next day of context.now(). Defaults to timedelta(hours=0).
             period_time_list (list[PeriodTime] | None): A list of period time parameters.
                 Weights default to 1. Period time to 14 days.
-            aggregation_mode (InputAggregationMode): The  aggregation mode of the number embedding.
+            aggregation_mode (InputAggregationMode): The  aggregation mode of the recency embedding.
                 Possible values are: maximum, minimum and average. Defaults to InputAggregationMode.INPUT_AVERAGE.
             negative_filter (float): The recency score of items that are older than the oldest period time.
                 Defaults to 0.0.
         """
         super().__init__(timestamp, Timestamp)
-        self.timestamp = SpaceFieldSet(self, self._field_set)
-        self.period_time_list: list[PeriodTime] = (
+        self._aggregation_config_type_by_mode = self.__init_aggregation_config_type_by_mode()
+        self.timestamp = SpaceFieldSet[int](self, set(timestamp if isinstance(timestamp, list) else [timestamp]))
+        recency_periods: list[PeriodTime] = (
             period_time_list
             if isinstance(period_time_list, list)
-            else (
-                [period_time_list]
-                if period_time_list is not None
-                else [DEFAULT_PERIOD_TIME]
-            )
+            else ([period_time_list] if period_time_list is not None else [DEFAULT_PERIOD_TIME])
         )
-        self.negative_filter = negative_filter
-        self.time_period_hour_offset = time_period_hour_offset
-        self.__aggregation_mode: InputAggregationMode = aggregation_mode
-        self.__run_parameter_checks()
-        self.__schema_node_map: dict[SchemaObject, RecencyNode] = {
+        self._aggregation_mode: InputAggregationMode = aggregation_mode
+        self._embedding_config = RecencyEmbeddingConfig(
+            int,
+            recency_periods,
+            time_period_hour_offset,
+            negative_filter,
+        )
+        self._transformation_config = self._init_transformation_config(
+            self._embedding_config, self._aggregation_mode, recency_periods
+        )
+        self._schema_node_map: dict[SchemaObject, EmbeddingNode[int, int]] = {
             field.schema_obj: RecencyNode(
-                SchemaFieldNode(field),
-                self.time_period_hour_offset,
-                self.period_time_list,
-                self.__aggregation_mode,
-                self.negative_filter,
+                parent=SchemaFieldNode(field),
+                transformation_config=self._transformation_config,
+                fields_for_identification=self.timestamp.fields,
             )
             for field in self.timestamp.fields
         }
         self._max_period_time_days = (
-            max(
-                self.period_time_list, key=lambda p: p.period_time
-            ).period_time.total_seconds()
+            max(self._embedding_config.period_time_list, key=lambda p: p.period_time).period_time.total_seconds()
             / 86400
         )
 
     @property
-    def _node_by_schema(self) -> Mapping[SchemaObject, Node[Vector]]:
-        return self.__schema_node_map
+    @override
+    def space_field_set(self) -> SpaceFieldSet:
+        return self.timestamp
 
     @property
     @override
-    def annotation(self) -> str:
+    def transformation_config(self) -> TransformationConfig[int, int]:
+        return self._transformation_config
+
+    def __init_aggregation_config_type_by_mode(
+        self,
+    ) -> dict[InputAggregationMode, type[AggregationConfig]]:
+        return {
+            InputAggregationMode.INPUT_AVERAGE: AvgAggregationConfig,
+            InputAggregationMode.INPUT_MINIMUM: MinAggregationConfig,
+            InputAggregationMode.INPUT_MAXIMUM: MaxAggregationConfig,
+        }
+
+    @property
+    @override
+    def _embedding_node_by_schema(
+        self,
+    ) -> dict[SchemaObject, EmbeddingNode[int, int]]:
+        return self._schema_node_map
+
+    @property
+    @override
+    def _annotation(self) -> str:
         return f"""The space encodes timestamps between now and now - {self._max_period_time_days} days.
         Older timestamps will have negative_filter similarity (assuming weight = 1).
         There is no differentiation between items older than {self._max_period_time_days} days.
@@ -137,57 +174,42 @@ class RecencySpace(Space):  # pylint: disable=too-many-instance-attributes
         Also take into account that max_period_time is {self._max_period_time_days} days".
         Favoring values from {self._max_period_time_days/2} days ago could best be achieved using positive weights
         lower than the other weights in the query.
-        Larger positive weights increase the effect on similarity compared to other spaces. Space weights do not
-        matter if there is only 1 space in the query.
-        Does not have input as querying will always happen using the utc timestamp of the system's NOW."""
+        Larger positive weights increase the effect on similarity compared to other spaces.
+        Does not have input as querying will always happen using the utc timestamp of the system's NOW.
+        Affected fields: {self.space_field_set.field_names_text}."""
 
     @property
     @override
     def _allow_empty_fields(self) -> bool:
         return True
 
-    def __run_parameter_checks(self) -> None:
-        if self.negative_filter > 0:
-            sum_weights: float = sum(param.weight for param in self.period_time_list)
+    @property
+    @override
+    def allow_similar_clause(self) -> bool:
+        return False
 
-            max_period_time: timedelta = max(
-                param.period_time for param in self.period_time_list
-            )
-            max_period_time_str = (
-                f"{max_period_time.days} days"
-                if max_period_time.days
-                else f"{round(max_period_time.total_seconds() / 3600, 2)} hours"
-            )
-            logger.warning(
-                "Positive negative_filter value was supplied. This will lead to "
-                "old items (older than max_period_time) having recency scores of "
-                "negative_filter.\nMeanwhile the largest recency score possible for the most "
-                "recent items is around the sum of weights, and the score at max_period_time will be 0. "
-                "\nUse with caution.",
-                negative_filter=self.negative_filter,
-                max_period_time=max_period_time_str,
-                sum_of_weights=sum_weights,
-            )
-
-        if any(param.weight < 0 for param in self.period_time_list):
-            logger.warning(
-                "Negative weight was supplied for some period_time_param. This can lead to very strange "
-                "recency score curves. Use with caution. \n"
-                "To better understand your recency scores use RecencyPlotter."
-                "It can be imported from `superlinked.evaluation.charts.recency_plotter`. \n"
-                "Check an example notebook at: https://github.com/superlinked/superlinked/blob/main"
-                "/notebook/combining_recency_and_relevance.ipynb. "
-            )
+    def _init_transformation_config(
+        self,
+        embedding_config: RecencyEmbeddingConfig,
+        aggregation_mode: InputAggregationMode,
+        recency_periods: Sequence[PeriodTime],
+    ) -> TransformationConfig[int, int]:
+        aggregation_config = self._aggregation_config_type_by_mode[aggregation_mode](int)
+        normalization_config = ConstantNormConfig(
+            math.sqrt(sum(period_time.weight**2 for period_time in recency_periods))
+        )
+        return TransformationConfig(
+            normalization_config,
+            aggregation_config,
+            embedding_config,
+        )
 
     @override
-    def _handle_node_not_present(self, schema: SchemaObject) -> RecencyNode:
+    def _create_default_node(self, schema: SchemaObject) -> EmbeddingNode[int, int]:
         named_function_node = NamedFunctionNode(NamedFunction.NOW, schema, int)
-        recency_node = RecencyNode(
-            named_function_node,
-            self.time_period_hour_offset,
-            self.period_time_list,
-            self.__aggregation_mode,
-            self.negative_filter,
+        default_node = RecencyNode(
+            parent=named_function_node,
+            transformation_config=self._transformation_config,
+            fields_for_identification=self.timestamp.fields,
         )
-        self.__schema_node_map[schema] = recency_node
-        return recency_node
+        return default_node
