@@ -24,9 +24,8 @@ from superlinked.framework.common.dag.context import ExecutionContext
 from superlinked.framework.common.data_types import Vector
 from superlinked.framework.common.exception import ValidationException
 from superlinked.framework.common.interface.has_length import HasLength
-from superlinked.framework.common.space.normalization import L2Norm
+from superlinked.framework.common.space.normalization.normalization import ConstantNorm
 from superlinked.framework.common.storage_manager.storage_manager import StorageManager
-from superlinked.framework.common.util.weight_arithmetics import WeightArithmetics
 from superlinked.framework.online.dag.default_online_node import DefaultOnlineNode
 from superlinked.framework.online.dag.evaluation_result import SingleEvaluationResult
 from superlinked.framework.online.dag.online_node import OnlineNode
@@ -40,12 +39,8 @@ class OnlineConcatenationNode(DefaultOnlineNode[ConcatenationNode, Vector], HasL
         parents: list[OnlineNode],
         storage_manager: StorageManager,
     ) -> None:
-        super().__init__(
-            node,
-            parents,
-            storage_manager,
-            ParentValidationType.AT_LEAST_ONE_PARENT,
-        )
+        super().__init__(node, parents, storage_manager, ParentValidationType.AT_LEAST_ONE_PARENT)
+        self._norm = ConstantNorm(self.node.create_normalization_config([1.0] * len(self.node.parents)))
 
     @property
     def length(self) -> int:
@@ -57,62 +52,37 @@ class OnlineConcatenationNode(DefaultOnlineNode[ConcatenationNode, Vector], HasL
         parent_results: list[dict[OnlineNode, SingleEvaluationResult]],
         context: ExecutionContext,
     ) -> Sequence[Vector | None]:
-        self.__check_evaluation_inputs(parent_results)
-        vectors = [
-            reduce(
-                lambda a, b: a.concatenate(b),
-                (
-                    WeightArithmetics.apply_vector_weight(
-                        result.value, parent.node_id, context
-                    )
-                    for parent, result in parent_result.items()
-                ),
-            )
-            for parent_result in parent_results
+        self._check_evaluation_inputs(parent_results)
+        vector_and_nodes_list: list[list[tuple[Vector, OnlineNode]]] = [
+            [(result.value, parent) for parent, result in parent_result.items()] for parent_result in parent_results
         ]
-        if context.is_query_context:
-            return vectors
-        l2_norm = L2Norm()
-        return [l2_norm.normalize(vector) for vector in vectors]
+        weighted_vectors = [
+            self._apply_weights_and_concatenate(vector_and_nodes, context) for vector_and_nodes in vector_and_nodes_list
+        ]
+        return [self._norm.normalize(vector) for vector in weighted_vectors]
 
-    def re_weight_vector(
+    def _apply_weights_and_concatenate(
         self,
-        vector: Vector,
+        vector_and_nodes: list[tuple[Vector, OnlineNode]],
         context: ExecutionContext,
     ) -> Vector:
-        parts = self._split_vector(vector)
-        vector = reduce(
-            lambda a, b: a.concatenate(b),
-            (
-                WeightArithmetics.apply_vector_weight(part, parent.node_id, context)
-                for part, parent in zip(parts, self.parents)
-            ),
-        )
-        return L2Norm().normalize(vector)
+        weighted_vectors = (vector * context.get_weight_of_node(parent.node_id) for vector, parent in vector_and_nodes)
+        vector = reduce(lambda a, b: a.concatenate(b), weighted_vectors)
+        return vector
 
-    def __check_evaluation_inputs(
+    def _check_evaluation_inputs(
         self,
         parent_results: list[dict[OnlineNode, SingleEvaluationResult]],
     ) -> None:
-        invalid_results = [
+        if any(
             result
             for parent_result in parent_results
             for result in parent_result.values()
             if not isinstance(result.value, Vector)
-        ]
-        if len(invalid_results) != 0:
-            raise ValidationException(
-                f"{self.class_name} can only process `Vector` inputs."
-            )
+        ):
+            raise ValidationException(f"{self.class_name} can only process `Vector` inputs.")
 
-    def _split_vector(self, vector: Vector) -> list[Vector]:
-        if vector is None:
-            vector = Vector([])
-        offset: int = 0
-        parts: list[Vector] = []
-        vector_value = vector.value
-        for parent in self.parents:
-            parent_length = cast(HasLength, parent).length
-            parts.append(Vector(vector_value[offset : offset + parent_length]))
-            offset += parent_length
-        return parts
+    def _split_vector(self, vector: Vector, parents_without_duplicates: Sequence[OnlineNode]) -> list[Vector]:
+        lengths = [cast(HasLength, parent).length for parent in parents_without_duplicates]
+        vectors: list[Vector] = vector.split(lengths)
+        return vectors
