@@ -13,14 +13,17 @@
 # limitations under the License.
 
 
+import json
+
 import numpy as np
 from beartype.typing import Any, Callable, cast
 
 from superlinked.framework.common.data_types import Vector
+from superlinked.framework.common.schema.blob_information import BlobInformation
 from superlinked.framework.common.storage.exception import EncoderException
-from superlinked.framework.common.storage.field import Field
-from superlinked.framework.common.storage.field_data import FieldData
-from superlinked.framework.common.storage.field_data_type import FieldDataType
+from superlinked.framework.common.storage.field.field import Field
+from superlinked.framework.common.storage.field.field_data import FieldData
+from superlinked.framework.common.storage.field.field_data_type import FieldDataType
 
 RedisEncodedTypes = str | float | int | list[float] | bytes
 
@@ -33,6 +36,7 @@ class RedisFieldEncoder:
             FieldDataType.DOUBLE: self._encode_double,
             FieldDataType.FLOAT_LIST: self._encode_float_list,
             FieldDataType.INT: self._encode_int,
+            FieldDataType.JSON: self._encode_json,
             FieldDataType.STRING: self._encode_string,
             FieldDataType.STRING_LIST: self._encode_string_list,
             FieldDataType.VECTOR: self._encode_vector,
@@ -42,16 +46,17 @@ class RedisFieldEncoder:
             FieldDataType.DOUBLE: self._decode_double,
             FieldDataType.FLOAT_LIST: self._decode_float_list,
             FieldDataType.INT: self._decode_int,
+            FieldDataType.JSON: self._decode_json,
             FieldDataType.STRING: self._decode_string,
             FieldDataType.STRING_LIST: self._decode_string_list,
             FieldDataType.VECTOR: self._decode_vector,
         }
 
-    def _encode_blob(self, blob: str) -> str:
-        return blob
+    def _encode_blob(self, blob: BlobInformation) -> str | None:
+        return blob.path
 
-    def _decode_blob(self, blob: bytes) -> str:
-        return blob.decode("utf-8")
+    def _decode_blob(self, blob: bytes) -> BlobInformation:
+        return BlobInformation(path=blob.decode("utf-8"))
 
     def _encode_double(self, double: float) -> float:
         return double
@@ -71,6 +76,12 @@ class RedisFieldEncoder:
     def _decode_int(self, int_: bytes) -> int:
         return int(int_)
 
+    def _encode_json(self, json_: dict[str, Any]) -> str:
+        return json.dumps(json_, ensure_ascii=True)
+
+    def _decode_json(self, json_: bytes) -> dict[str, Any]:
+        return json.loads(json_.decode("utf-8")) if json_ else {}
+
     def _encode_string(self, string: str) -> str:
         return string
 
@@ -93,31 +104,25 @@ class RedisFieldEncoder:
 
     def _decode_vector(self, vector: bytes) -> Vector:
         if not isinstance(vector, bytes):
-            raise NotImplementedError(
-                f"Cannot decode non-bytes type vector, got: {type(vector)}"
-            )
+            raise NotImplementedError(f"Cannot decode non-bytes type vector, got: {type(vector)}")
         return Vector(np.frombuffer(vector, np.float32).tolist())
 
     def encode_field(self, field: FieldData) -> RedisEncodedTypes:
         if encoder := self._encode_map.get(field.data_type):
             return encoder(field.value)
-        raise EncoderException(
-            f"Unknown field type: {field.data_type}, cannot encode field."
-        )
+        raise EncoderException(f"Unknown field type: {field.data_type}, cannot encode field.")
 
     def decode_field(self, field: Field, value: bytes) -> FieldData:
         if decoder := self._decode_map.get(field.data_type):
             return FieldData.from_field(field, decoder(value))
-        raise EncoderException(
-            f"Unknown field type: {field.data_type}, cannot decode field."
-        )
+        raise EncoderException(f"Unknown field type: {field.data_type}, cannot decode field.")
 
-    def convert_bytes_keys(self, data: dict[bytes, Any]) -> dict[str, Any]:
+    def convert_bytes_keys_dict(self, data: dict[bytes, Any]) -> dict[str, Any]:
+        return {self._decode_string(cast(bytes, key)): self.convert_bytes_keys(value) for key, value in data.items()}
+
+    def convert_bytes_keys(self, data: Any) -> Any:
         if isinstance(data, dict):
-            return {
-                self._decode_string(cast(bytes, key)): self.convert_bytes_keys(value)
-                for key, value in data.items()
-            }
+            return self.convert_bytes_keys_dict(data)
         if isinstance(data, list):
             return [self.convert_bytes_keys(item) for item in data]
         if isinstance(data, tuple):
